/*
 * wpm.d - Measure words per minute of typing.
 *         Written in DTrace (Solaris 10 3/05).
 *
 * $Id: wpm.d 52 2007-09-24 04:28:01Z brendan $
 *
 * This script assumes that keystrokes arrive one at a time on STDIN. This
 * isn't the case for all processes that read keyboard input (eg, sh).
 *
 * COPYRIGHT: Copyright (c) 2007 Brendan Gregg.
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * 05-Aug-2007	Brendan Gregg	Created this.
 * 07-Nov-2008	Tom Erickson	(adapted for Chime)
 */

/*
 * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
 */

inline int STDIN = 0;

enum tracing_state {
	BEGIN,
	TRACING
};

dtrace:::BEGIN
{
	state = BEGIN;
	keys = 0;
	words = 0;
	wordsize = 0;
	countdown = 5;
	last = 0;
	printf("%s %d", "TIME", countdown);
}

profile:::tick-1sec
/--countdown >= 0/
{
	printf("%s %d", "TIME", countdown);
}

profile:::tick-1sec
/state == BEGIN && countdown == -1/
{
	state = TRACING;
	countdown = 60;
	printf("%s %d %d", "TIME", countdown, 1);
}

syscall::read:entry
/state == TRACING && execname == $$1 && arg0 == STDIN/
{
	self->buf = arg1;
}

syscall::read:return
/self->buf/
{
	keys++;
	printf("%s %d", "KEYS", keys);
	wordsize++;
	this->key = stringof(copyin(self->buf, arg0));
	last = timestamp;
}

syscall::read:return
/self->buf && (this->key == " " || this->key == "\n" || this->key == "\r") &&
    wordsize == 1/
{
	/* recurring space */
	wordsize = 0;
	self->buf = 0;
}

syscall::read:return
/self->buf && (this->key == " " || this->key == "\n" || this->key == "\r")/
{
	words++;
	printf("%s %d", "WORDS", words);
	@sizes = lquantize(wordsize - 1, 0, 32, 1);
	wordsize = 0;
}

syscall::read:return
/self->buf/
{
	self->buf = 0;
}

profile:::tick-1sec
/state == TRACING && countdown == -1/
{
	printf("%s %d %d", "WORDS", words, 1);
	printf("%s %d %d", "TIME", countdown, 2);
	exit(0);
}
